
import sys
import time
import numpy as np
import soundfile as sf
from numpy.lib.stride_tricks import sliding_window_view

from harkpython import harkbasenode

class HarkNode(harkbasenode.HarkBaseNode):
    def __init__(self):
        self.outputNames=("AUDIO", "NOT_EOF",)
        self.outputTypes=("matrix_float", "prim_bool",)
        self.frame_size = 512 # same as LENGTH parameter of AudioStreamFromWave
        self.advance = 160 # same as ADVANCE parameter of AudioStreamFromWave
        self.frames = np.array([], dtype=np.float32)
        self.nframes = 0
        self.frame_counter = 0
        self.initialized = False

    def load_file(self, filename=""):
        if filename:
            audio, rate = sf.read(filename, dtype=np.float32)
            audio *= 2**15 # e.g.) re-scaled floating point data for the 16bit microphone
            nch = audio.shape[1]
            self.frames = sliding_window_view(audio, self.frame_size, axis=0)[::self.advance, :, :]
            self.nframes = self.frames.shape[0]

    def load_files(self, filenames=[]):
        self.frames = np.array([], dtype=np.float32)
        self.nframes = 0
        for filename in filenames if isinstance(filenames, list) else []:
            if isinstance(filename, str):
                audio, rate = sf.read(filename, dtype=np.float32)
                audio *= 2**15 # e.g.) re-scaled floating point data for the 16bit microphone
                nch = audio.shape[1]
                frames = sliding_window_view(audio, self.frame_size, axis=0)[::self.advance, :, :]
                if self.frames.shape[1] == frames.shape[1] and self.frames.shape[2] == self.frames.shape[2]:
                    self.frames = np.concatenate([self.frames, frames])
                else:
                    # It was detected that the file was read for the first time (case of self.frame.size == 0) or
                    # that the number of channels or length has changed from the previous file.
                    self.frames = sliding_window_view(audio, self.frame_size, axis=0)[::self.advance, :, :]
                self.nframes = self.frames.shape[0]

    def calculate(self):
        if not self.initialized:
            #txt_file_path = "confirm.txt"
            #with open(txt_file_path, "r") as f:
            #    content = f.read()
            #if content == 1:
            #    self.load_file("realtime1.wav")
            #else:
            #    self.load_file("tsp_d120.wav")
            self.load_file("2SPK-ja.wav")
            self.initialized = True

        if self.frame_counter >= self.nframes:
            self.frame_counter = 0

        if self.nframes > 0:
            self.outputValues["AUDIO"] = self.frames[self.frame_counter]
            self.outputValues["NOT_EOF"] = True
        else:
            self.outputValues["AUDIO"] = np.array([[]], dtype=np.float32)
            self.outputValues["NOT_EOF"] = False

        self.frame_counter += 1

